/*
 * *****************************************************************
 * *                                                               *
 * *   Copyright 2003 Hewlett-Packard Development Company, L.P.    *
 * *                                                               *
 * *   The software contained on this media is proprietary to and  *
 * *   embodies the  confidential  technology  of Hewlett-Packard  *
 * *   Development   Company.  Possession,  use,  duplication  or  *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license  from  Hewlett-Packard  *
 * *   Company.                                                    *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: rdg.h,v $ $Revision: 1.1.18.1 $ (DEC) $Date: 2001/09/21 19:52:54 $
 */

/*
 * rdg.h - defines and structures used by applications
 */
 
#define RDG_VERS		39

/*
 * Context handle for a RDG session
 */

typedef unsigned long		RdgCtxHdl;

/*
 * Endpoint handle definition
 */

typedef struct _RdgEpHdl {
    unsigned long		node:8;
    unsigned long		incarn:8;
    unsigned long		reserved:16;
    unsigned long		port:32;
} RdgEpHdl;

/*
 * Buffer region handle definition
 */

typedef struct _RdgBufHdl {
    unsigned long		node:8;
    unsigned long		incarn:8;
    unsigned long		reserved:16;
    unsigned long		id:32;
} RdgBufHdl;

/*
 * Opaque object handle definition
 */

typedef struct _RdgObjHdl {
    unsigned long		node:8;
    unsigned long		incarn:8;
    unsigned long		reserved:16;
    unsigned long		id:32;
} RdgObjHdl;

/*
 * I/O descriptor. We make it quadword aligned for more efficient copying.
 */

typedef struct {
    unsigned long		size;
    unsigned int		state;
    unsigned int		version;
    union {
	RdgEpHdl		ep;
        RdgBufHdl		buf;
        RdgObjHdl		obj;
    } hdl;
    unsigned int		type;
    unsigned int		flags;
    void			*addr;
    unsigned long		bufOffset;
    unsigned long		bytesSent;
    unsigned int		result;
    unsigned int		seq;
    long			cmpltParam;
    unsigned long		userArg1;
    unsigned long		userArg2;
    unsigned long		chksum;
    int			        niovec;
    int			        reserved[7];
} RdgIoDesc;

/*
 * I/O descriptor types
 */

#define RDG_MSG_SND		0x1
#define RDG_MSG_RCV		0x2
#define RDG_BUF_SND		0x4
#define RDG_BUF_RCV		0x8
#define RDG_TUCKASIDE          0x10

/*
 * I/O descriptor flags
 */

#define RDG_BROADCAST_ON_BADEP	0x1
#define RDG_ADDR_VEC		0x2
#define RDG_INC_INCARNATION     0x4    /* Thought of using RDG_INC_INC  ;-) */
#define RDG_DO_NOT_QUEUE        0x8
#define RDG_AUTO_WIRE          0x10 
#define RDG_NOWIRE             0x20  /* these buffers never get wired because 
                                      * they are in a special system memory 
				      * region
				      */
#define RDG_TELL_IF_DONE       0x40


/*
 * Flags to RdgEpCreate
 */

#define RDG_ADDR_HINT		0x1

/*
 * Flags to RdgBufCreate
 */

#define RDG_ADDR_HINT		0x1

/*
 * Flags to RdgIoPoll
 */

#define RDG_IO_PEEK		0x1
#define RDG_RCV_ONLY		0x2
#define RDG_SND_ONLY		0x4

/*
 * System limits.
 */

#define RDG_MAX_OBJS		10240

/*
 * Status values
 */

#define RDG_OK			0
#define RDG_BLOCK		1
#define RDG_OBJLIMIT	        2
#define RDG_BADEP		3
#define RDG_ADDRINUSE		4
#define RDG_NOREMOTE		5
#define RDG_MSGSIZE		6
#define RDG_KERNINIT		7
#define RDG_INTR		8
#define RDG_NOMEM		9
#define RDG_BADREQ		10
#define RDG_PROCINIT		11
#define RDG_UNUSED	        12
#define RDG_BADCTX	        13
#define RDG_BADADDR	        14
#define RDG_BADBUF		16
#define RDG_BADHOST		17
#define RDG_BUFSIZE		18
#define RDG_BADOBJ		19
#define RDG_BADVERS		20
#define RDG_QUEUED              21
#define RDG_ALREADY_DONE        22
