/*
 * *****************************************************************
 * *                                                               *
 * *   Copyright 2003 Hewlett-Packard Development Company, L.P.    *
 * *                                                               *
 * *   The software contained on this media is proprietary to and  *
 * *   embodies the  confidential  technology  of Hewlett-Packard  *
 * *   Development   Company.  Possession,  use,  duplication  or  *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license  from  Hewlett-Packard  *
 * *   Company.                                                    *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: user.h,v $ $Revision: 4.3.175.3 $ (DEC) $Date: 2002/08/27 14:21:56 $
 */ 
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * Copyright (C) 1988,1989 Encore Computer Corporation.
 * All Rights Reserved.
 *
 * Property of Encore Computer Corporation.
 * This software is made available solely pursuant to the terms of
 * a software license agreement which governs its use. Unauthorized
 * duplication, distribution or sale are strictly prohibited.
 *
 */
/* 
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * Copyright (c) 1988 Carnegie-Mellon University
 * Copyright (c) 1987 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */
/*
 * OSF/1 Release 1.2
 */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef	_SYS_USER_H_
#define _SYS_USER_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/secdefines.h>
#include <mach/boolean.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/namei.h>
#include <sys/ucred.h>
#include <sys/sem.h>
#include <sys/audit.h>
#include <sys/unix_defs.h>
#include <kern/lock.h>
#include <kern/queue.h>
#ifdef	_KERNEL
#include <sys/siginfo.h>
#include <kern/event.h>
#include <sys/security.h>
#include <machine/nxm.h>
#endif	/* _KERNEL */

#if defined (__cplusplus)
struct proc;
struct file;
struct exit_actn;
#endif

/*
 * Per process structure containing data that
 * isn't needed in core when the process is swapped out.
 */

/*
 * V5.0, the maximum login name size is 64 (up from 12).  By default, the
 * user still gets 12 but that can be bumped via sysconfigtab (generic)
 * by setting the global login_name_max.
 */
#define MAXLOGNAME      64
#define	DEFAULT_LOGNAME	12
#define	COMPAT_LOGNAME	12

#define MAXCOMLEN	16	/* <= MAXNAMLEN, >= sizeof(ac_comm) */

struct flag_field {
	int	fi_flag;
	udecl_simple_lock_data(,*fi_lock) /* unused */
};

#define NOFILE_IN_U   64 
#define U_FE_ALLOC_SIZE 8 		/* allocation size of uarea open file */
#define U_FE_ARRAY_SIZE ((NOFILE_IN_U + U_FE_ALLOC_SIZE - 1)/U_FE_ALLOC_SIZE)

#define U_FE_OF_ALLOC_SIZE 256 		/* allocation size for overflow area */

#define NUM_LVL1_IN_LVL0	(sizeof(long) * NBBY)
#define NUM_FD_IN_LVL1		(sizeof(long) * NBBY)
#define MAX_NUM_LVL0_ENTRIES  \
	  (NEW_OPEN_MAX_SYSTEM / (NUM_LVL1_IN_LVL0 * NUM_FD_IN_LVL1))

#ifdef __alpha
typedef enum {
	SPEC_ACTION_RAISE,
	SPEC_ACTION_IGNORE,
	SPEC_ACTION_CALL_HANDLER,
	SPEC_ACTION_INITIALIZE,
	SPEC_FAULT_IGNORE,
	SPEC_FAULT_RAISE,
	SPEC_ACTION_INITIALIZE_FB_FIXUP
} spec_action;

struct uuspeculative {	/* speculative execution data */
	int		sp_version;
	spec_action	sp_segv_action;
	spec_action	sp_fpe_action;
	vm_offset_t	sp_handler_addr;
};
#endif /* __alpha */

#if	(defined(_KERNEL) || defined(SHOW_UTT))

/*
 *      Forward Declarations of structures at file scope to make 
 *      pre-ANSI C++ happy.
 */
struct processor;
struct np_uthread;
struct utask;
struct task;
struct event;
struct execops;

/*
 *	Per-thread U area.
 *
 *	It is likely that this structure contains no fields that must be
 *	saved between system calls.  Note that the size of this structure
 *	must not be changed in minor releases because it affects binary
 *	compatibility with kernel layered products (due to the arrangement
 *	of the stack_layout structure).
 */
struct uthread {
/* namei & co. */
	struct unameicache {		/* last successful directory search */
		int nc_prevoffset;	/* offset at which last entry found */
		ino_t nc_inumber;	/* inum of cached directory */
		dev_t nc_dev;		/* dev of cached directory */
		time_t nc_time;		/* time stamp for cache entry */
	} uu_ncache;
	struct	nameidata uu_nd;
	int	reserved;
	/*
	 * Member uu_upcf is written at interrupt level.
	 * Give it its own longword, to avoid any locking
	 * overlap (if anything here needed locking, which
	 * nothing currently does).
	 */
	int     uu_upcf;

#ifdef __alpha

/* per thread ieee floating point state */
	ulong_t	uu_ieee_fp_trap_pc;
	ulong_t	uu_ieee_fp_trigger_sum;
	uint_t	uu_ieee_fp_trigger_inst;

	boolean_t uu_asmp;	/* AdvFS ASMP indicator */

        /* the following ieee variables are set via hal_sysinfo(2) */
	ulong_t	uu_ieee_fp_control;
	ulong_t	uu_ieee_set_state_at_signal;
	ulong_t	uu_ieee_fp_control_at_signal;
	ulong_t	uu_ieee_fpcr_at_signal;
#else /* __alpha */
	boolean_t uu_asmp;	/* AdvFS ASMP indicator */
#endif /* __alpha */

/* audit information */ 
	struct sysent *uu_callp;	/* system call table entry */
	int     uu_event;		/* audit event             */
	int     uu_vno_indx;		/* # of vno_{dev,num} used */
	dev_t   uu_vno_dev[AUD_VNOMAX];	/* vnode dev's referenced by cur proc */
	ino_t   uu_vno_num[AUD_VNOMAX];	/* vnode num's referenced by cur proc */
	ino_t   uu_vno_pdir[AUD_VNOMAX]; /* vnode num's parent dir referenced by cur proc */
	ushort  uu_vno_aud[AUD_VNOMAX];	/* vnode audit-mode bits   */
	ushort  uu_vno_mode[AUD_VNOMAX];/* vnode mode flag         */
	u_int   uu_set_uids_snap;       /* snapshot of set_uids    */
	pid_t   uu_target_pid;          /* target of mach ipc op   */

#if SEC_MAC
	tag_t	uu_obj_slevel[AUD_VNOMAX]; /* vnode/object sensitivity level (SL) */
#endif
#if SEC_ILB
	tag_t	uu_obj_ilevel[AUD_VNOMAX];  /* vnode/object information level (IL) */
#endif
#if SEC_BASE
	ushort	  uu_sec_error;		    /* security related errno */
	ushort	  uu_last_fail_priv;	    /* last priv that failed and */
	privvec_t uu_privs_used;	    /* history of privs used */
#endif
#if SEC_PRIV || SEC_ARCH
	sec_eventvec_t	uu_sec_event_hist;  /* recent security event history
					     * mask.  flag bits accessed
					     * SE_ symbols as offsets (from
					     * security.h).  */
#endif

        sigset_t uu_oldmask;            /* saved mask from before sigpause */

	struct uuprof uu_prof;
#ifdef _KERNEL
	struct nxm_pth_state *uu_proflast;/* last nxm thread to use uu_prof */
#endif
	stack_t  uu_sigstack;		/* sp & on stack state variable */

/* Cluster related uthread fields.  */
	/*
	 * uu_copyio_ctl must be first short of this int.
	 */
	ushort_t  uu_copyio_ctl;	/* copy[in,out][str]() control */
	char	  uu_icsprio;		/* ICS priority */
	uchar_t	  uu_filler;		/* Pad to uint_t */
	boolean_t uu_remotecopy;	/* CSPECFS related support */
	uint_t	  uu_remotenode;	/* CSPECFS related support */
	pid_t	  uu_remotepid;		/* CSPECFS related support */

	long	uu_devpoll_mode;	/* signal select routines we are 
					   being called on behalf of devpoll */

/* NOTE!!!
 * Don't add anything past this. These elements are at the end
 * of the structure to force cache alignments
 */
	struct rad *uu_rad;
	struct np_uthread *np_uthread;
	struct utask *utask;
	struct proc *proc;		/* pointer to proc structure */
	struct thread *thread;		/* quick lookup for current_thread() */
	struct task *task;		/* quick lookup for current_task() */
	struct processor *processor;	/* current_processor() */
	int	cpu_number;		/* cpu_number() */
	int	disable_preemption;	/* stop preemptions */
};

/*
 * Non-pageable per-thread U area.
 *
 * All the data here may be accessed on behalf of threads other
 * than the current thread, so must always be resident.
 */
struct np_uthread {
    /* 64-byte alignment */
	long	*uu_ar0;		/* address of users saved R0 */

/* thread exception handling */
	int	uu_code;		/* ``code'' to trap */
	int	uu_cursig;		/* signal to take */

/* per thread signal state */
	int	uu_tsig;		/* thread signal for upcalls */
	int	uu_sflags;		/* origin of signal */
	sigset_t uu_sigwaitmask;	/* signals waiting for */
#ifdef  _KERNEL
        struct event uu_select_event;	/* select waits on this */

    /* 64-byte alignment */
	queue_head_t uu_sigqueue;	/* queue of sigqueue structs */
	sigqueue_t uu_curinfo;		/* curinfo for exc. */ 
	struct thread_rusage uu_tru;
	int	spare1;

    /* 64-byte alignment */
	struct ushared_state *uu_sptr;
	struct ushared_state uu_share;

    /* 64-byte alignment */
#define MAX_TSD_SLOTS			8
	void *uu_tsd[MAX_TSD_SLOTS];	/* thread-specific data */
#endif
};

struct utask_nd {
	struct vnode *utnd_cdir;	/* current directory */
	struct vnode *utnd_rdir;	/* root directory of current process */
};

struct  ufile_entry {
	struct file *ufe_ofile;			/* pointer to file structure */
	struct devpoll_entry *ufe_dpe;       /* devpoll entry */
	struct socket_sel_queue *ufe_so_sel; /* socket select entry */
	int	ufe_unused;
	int	ufe_oflags;
#define UF_EXCLOSE	0x1			/* auto-close on exec */
#define UF_MAPPED	0x2			/* mapped from device */
#define UF_RESERVED_WAIT 0x4			/* thread waiting for slot in dup2 */

	udecl_simple_lock_data(,ufe_ofile_lock)	/* lock controls structure fields */
	long	pad[3];				/* make structure 64 bytes */
};

/*
 * Structures associated with the per-process open file table.
 */
struct ufile_state {
	udecl_simple_lock_data(,uf_ofile_lock)
	int	utask_need_to_lock;	/* set if utask locks needed */
	int	uf_first_available;	/* low water mark for available
						file descriptors */
	int	uf_lastfile;		/* high-water mark of uf_ofile */
	u_int	uf_of_count;		/* number of overflow entrys */

	u_int	uf_flags;

#define UF_TABLE_SHRINK_PENDING 1	/* thread waiting to do file table
					 * shrink... wait for uf_references to
					 * go to zero
					 */
#define UF_TABLE_SHRINK_WAIT	2	/* thread waiting for task open file
					 * table shrink operation to finish.
					 */
#define UF_TABLE_FD_WAIT	4	/* wait for free fdbit to be cleared */

	u_int	uf_references;		/* used to block table shrink */

	/*
	 * Open file bit arrays.  Bit set means that the file descriptor
	 * has been allocated.  If no fd > NOFILE_IN_U have been allocated,
	 * uf_popen_bits_lvlx points to uf_open_bits_lvlx.  This depends
	 * on NOFILE_IN_U never being > 64.  If the table expands,
	 * (uf_of_count > 0) uf_popen_bits_lvlx points to KALLOC'ed buffers.
	 */
	u_long	uf_open_bits_lvl0;
	u_long	uf_open_bits_lvl1;
	u_long	*uf_popen_bits_lvl0;
	u_long	*uf_popen_bits_lvl1;

	/*
	 * If greater than NOFILE_IN_U file descriptors are allocated,
	 * uf_ofile_of and uf_pofile_of are used to reference the KALLOC'ed
	 * buffers which store the additional entries.
	 */
	struct ufile_entry	*uf_entry[U_FE_ARRAY_SIZE];
	struct ufile_entry	**uf_of_entry;/* Pointer to KALLOC'ed buffer */
};


struct socket_sel_queue {
	struct queue_entry	links;
	int			msel;	/* set if multiple threads selecting */
	int			mbz;	/* maker to tell sel_queue form so_* */
	struct event		*event; /* event to wake-up */
	struct ufile_state	*ufile;	/* file state of task involved */
	struct task		*task;
	struct ufile_entry	*fe;
};

/*
 *	Per-task U area - global process state.
 */
struct utask {
    /* 64-byte-aligned section of read-mostly (rarely modified) data */
	char		uu_comm[MAXCOMLEN + 1];
	int		uu_maxuprc;	/* max processes per UID */
	char		*uu_logname;	/* login name, if available */
	char		*uu_argp;	/* pointers to args and env */
	char		*uu_envp;
	int		uu_arg_size;	/* and associated lengths */
	int		uu_env_size;
	struct timeval	uu_start;	/* process start time */

    /* 64-byte-aligned section of memory management data */
	caddr_t		uu_text_start;	/* text starting address */
	caddr_t		uu_data_start;	/* data starting address */
	caddr_t		uu_stack_start;	/* stack starting address */
	caddr_t		uu_stack_end;	/* stack ending address */
	size_t		uu_tsize;	/* text size (clicks) */
	size_t		uu_dsize;	/* data size (clicks) */
	size_t		uu_ssize;	/* stack size (clicks) */
	boolean_t	uu_stack_grows_up; /* stack grows at high end? */
	int		uu_lflags;	/* process lock flags */
#define UL_TXTLOCK	1
#define UL_DATLOCK	2
#define UL_STKLOCK	4
#define UL_PROLOCK	8

    /* 64-byte-aligned section of file descriptor management data */
	struct ufile_state uu_file_state; /* open file information */
	char		*uu_sel_cache;
	long		uu_sel_size;
	long		uu_ioch;	/* # of chars read/written */
	mode_t		uu_cmask;	/* mask for file creation */
	u_int		uu_fd_newmax;	/* extended descriptor support */
	long		spare1;
	long		spare2;
	long		spare3;

    /* 64-byte-aligned section of some of the handy-locked data */
	udecl_simple_lock_data(,uu_handy_lock) /* handy lock for misc. data */
	u_int		uu_set_uids;	/* uid change count */
	int		uu_shmsegs;	/* # attached shared-memory-segments */
	struct flag_field uu_acflag;	/* accounting flags */
	struct utask_nd uu_utnd;	/* current/chroot directory vnodes */
	long		spare4;
	long		spare5;

    /* 64-byte-aligned section of resource limit data */
	struct rlimit	uu_rlimit[RLIM_NLIMITS];

    /* 64-byte-aligned section of resource usage statistics */
	struct rusage	uu_ru;		/* stats for this proc */
	struct rusage	uu_cru;		/* sum of stats for reaped children */

    /* 64-byte-aligned section of the rest */
	lock_data_t	uu_addr_lock;	/* user addr space lock */
	struct sem_undo *uu_semundo;	/* semaphore undo structure */
	char		*uu_coredir;	/* pointer to core path */

	struct execops	*uu_mgr;	/* file format manager */
	void		*uu_handle;	/* handle for that manager */
	struct compat_mod *uu_compat_mod; /* compat module control block */
	u_int		uu_sigresvcnt;	/* reserved mesq notif. queue count */
	int		spare7;
	long		uu_task_rq_wait_sum; /* runq wait sum exited threads */
	struct exit_actn *uu_exitp;	/* LMF actions to do at process exit */
#ifdef __alpha
	struct uuspeculative *uu_spec;	/* speculative execution */
	void		(*uu_fbhandler)();
#endif /* __alpha */

	u_int		uu_auditmask[AUDIT_INTMASK_LEN]; /* auditmask */
	u_int		uu_audit_cntl;	/* audit control flag */
	long		spare8[5];
};

#endif	/* (defined(_KERNEL) || defined(SHOW_UTT)) */

#include <machine/pcb.h>
struct user {
	struct	pcb u_pcb;
	struct	proc *u_procp;		/* pointer to proc structure */
	long	*u_ar0;			/* address of users saved R0 */
	char	u_comm[MAXCOMLEN + 1];

/* ssycall parameters, results and catches */
	int	u_arg[8];		/* arguments to current system call */
					/* now only used for signal */

/* 1.1 - processes and protection */
	char	u_logname[MAXLOGNAME];	/* login name, if available */

/* 1.2 - memory management */
	/* u_tsize must follow u_logname - TBL_UAREA bin compat */
	size_t	u_tsize;		/* text size (clicks) */
	size_t	u_dsize;		/* data size (clicks) */
	size_t	u_ssize;		/* stack size (clicks) */
	caddr_t	u_text_start;		/* text starting address */
	caddr_t	u_data_start;		/* data starting address */
	caddr_t	u_stack_start;		/* stack starting address */
	caddr_t	u_stack_end;		/* stack ending address */
	int	u_stack_grows_up;	/* stack grows at high end? */
	time_t	u_outime;		/* user time at last sample */

/* 1.3 - signal management */
	sig_t	u_signal[32+1];	/* disposition of signals */
	sigset_t u_sasigmask[32+1];	/* signals to be blocked */
	sigset_t u_sigonstack;		/* signals to take on sigstack */
	sigset_t u_sigintr;		/* signals that interrupt syscalls */
	sigset_t u_oldmask;		/* saved mask from before sigpause */
	int	u_code;			/* ``code'' to trap */
	stack_t u_sigstack;		/* sp & on stack state variable */
#define u_onstack	u_sigstack.ss_flags
#define u_sigsflags	u_sigstack.ss_flags
#define u_sigsp		u_sigstack.ss_sp
#define u_sigssz	u_sigstack.ss_size

/* 1.4 - descriptor management */
/* This information must exactly match ufile_state structure */
	struct	file *u_ofile[NOFILE_IN_U];/* file structs of open files */
	char	u_pofile[NOFILE_IN_U];	/* per-process flags of open files */
	int	u_lastfile;		/* high-water mark of u_ofile */
	/*
	 * If greater than NOFILE_IN_U file descriptors are allocated,
	 * uf_ofile_of and uf_pofile_of are used to reference the KALLOC'ed
	 * buffers which store the additional entries.
	 */
	u_int   u_of_count;
	struct  file    **u_ofile_of; /* Pointer to KALLOC'ed buffer */
	char            *u_pofile_of; /* Pointer to KALLOC'ed buffer */
#define UF_EXCLOSE	0x1		/* auto-close on exec */
#define UF_MAPPED	0x2		/* mapped from device */
				/* These definitions must match utask_nd */
	struct vnode *u_cdir;	/* current directory */
	struct vnode *u_rdir;	/* root directory of current process */
	short	u_cmask;		/* mask for file creation */

/* 1.5 - timing and statistics */
	struct	rusage u_ru;		/* stats for this proc */
	struct	rusage u_cru;		/* sum of stats for reaped children */
	struct	itimerval u_timer[3];
	int	u_XXX[3];
	struct	timeval u_start;
	struct	flag_field u_acflag;

	struct uuprof u_prof;
	u_short	u_maxuprc;		/* max processes per UID (per tree) */

/* 1.6 - resource controls */
	struct	rlimit u_rlimit[RLIM_NLIMITS];

/* namei & co. */
	struct nameicache {		/* last successful directory search */
		int nc_prevoffset;	/* offset at which last entry found */
		ino_t nc_inumber;	/* inum of cached directory */
		dev_t nc_dev;		/* dev of cached directory */
		time_t nc_time;		/* time stamp for cache entry */
	} u_ncache;
	struct	nameidata u_nd;
#define UL_TXTLOCK	1
#define UL_DATLOCK	2

/* compatability module pointer */
	struct compat_mod *u_compat_mod;

/* pointer to chain of LMF actions to do at process exit */
	struct exit_actn *u_exitp;     /* _LMF_ */
	int	u_stack[1];
};

/* u_error codes */
#include <sys/errno.h>

#ifdef	_KERNEL
#include <kern/thread.h>

#ifndef	u
#ifdef	multimax
extern struct u_address	active_uareas[NCPUS];
#define u       (active_uareas[cpu_number()])
#else	/* multimax */
#ifdef	balance
#define u	(*(struct u_address *) 0x40)
#else	/* balance */
#ifndef __digital__ /* need to know size of uthread & pcb before def. u */
#define u	(current_thread()->u_address)
#endif	/* __digital__ */
#endif	/* balance */
#endif	/* multimax */
#endif	/* u */
#ifdef __alpha
#define	u_ieee_fp_trigger_sum		uu_ieee_fp_trigger_sum
#define	u_ieee_fp_trigger_inst		uu_ieee_fp_trigger_inst
#define	u_ieee_fp_trap_pc		uu_ieee_fp_trap_pc
#define u_ieee_fp_control		uu_ieee_fp_control
#define u_ieee_set_state_at_signal	uu_ieee_set_state_at_signal
#define u_ieee_fp_control_at_signal	uu_ieee_fp_control_at_signal
#define u_ieee_fpcr_at_signal		uu_ieee_fpcr_at_signal
#endif /* __alpha */
#ifdef __digital__
#define u_procp				proc
#define u_nd				uu_nd
#define u_asmp				uu_asmp
#define u_upcf				uu_upcf
#define u_ncache			uu_ncache
#define u_callp				uu_callp
#define u_event				uu_event
#define u_vno_indx			uu_vno_indx
#define u_vno_dev			uu_vno_dev
#define u_vno_num			uu_vno_num
#define u_vno_pdir			uu_vno_pdir
#define u_vno_aud			uu_vno_aud
#define u_vno_mode			uu_vno_mode
#define u_set_uids_snap			uu_set_uids_snap
#define u_target_pid			uu_target_pid
#define u_oldmask			uu_oldmask
#define u_prof				uu_prof
#define u_proflast			uu_proflast
#define u_sigstack			uu_sigstack
#define u_onstack			u_sigstack.ss_flags
#define u_sigsflags			u_sigstack.ss_flags
#define u_sigsp				u_sigstack.ss_sp
#define u_sigssz			u_sigstack.ss_size
#if SEC_MAC
#define u_obj_slevel			uu_obj_slevel
#endif /* SEC_MAC */
#if SEC_ILB
#define u_obj_ilevel			uu_obj_ilevel
#endif /* SEC_ILB */
#if SEC_BASE
#define u_sec_error			uu_sec_error
#define u_privs_used			uu_privs_used
#define u_last_fail_priv		uu_last_fail_priv
#endif /* SEC_BASE */
#if SEC_PRIV || SEC_ARCH
#define u_sec_event_hist		uu_sec_event_hist
#endif /* SEC_PRIV || SEC_ARCH */
#define u_copyinctl			uu_copyio_ctl
#define u_copyio_ctl			uu_copyio_ctl
#define u_icsprio			uu_icsprio
#define u_remotecopy			uu_remotecopy
#define u_remotepid			uu_remotepid
#define u_remotenode			uu_remotenode
#define u_rad				uu_rad
#define u_devpoll_mode			uu_devpoll_mode
#else /* ! __digital__ */
#define u_nd		uthread->uu_nd
#define u_upcf		uthread->uu_upcf
#define u_ncache	uthread->uu_ncache
#define u_event		uthread->uu_event
#define u_vno_indx	uthread->uu_vno_indx
#define u_vno_dev	uthread->uu_vno_dev
#define u_vno_num	uthread->uu_vno_num
#define u_vno_pdir	uthread->uu_vno_pdir
#define u_vno_aud	uthread->uu_vno_aud
#define u_vno_mode	uthread->uu_vno_mode
#define u_set_uids_snap	uthread->uu_set_uids_snap
#define u_target_pid	uthread->uu_target_pid
#define u_oldmask	uthread->uu_oldmask
#define u_prof		uthread->uu_prof
#define u_sigstack	uthread->uu_sigstack
#define u_onstack	u_sigstack.ss_flags
#define u_sigsflags	u_sigstack.ss_flags
#define u_sigsp		u_sigstack.ss_sp
#define u_sigssz	u_sigstack.ss_size
#define	u_copyinctl	uthread->uu_copyio_ctl
#define	u_copyio_ctl	uthread->uu_copyio_ctl
#define u_icsprio	uthread->uu_icsprio
#define u_remotecopy	uthread->uu_remotecopy
#define u_remotepid	uthread->uu_remotepid
#define u_remotenode	uthread->uu_remotenode
#endif /* __digital__ */

/*
 * Cluster related CSPECFS support.
 */
#define IS_REMOTE_COPY(up)		((up)->u_remotecopy == TRUE)
#define SET_REMOTE_COPY(up, val)	(up)->u_remotecopy = (val)
#define SET_REMOTE_PID(up, val)		(up)->u_remotepid = (val)
#define GET_REMOTE_PID(up)		(up)->u_remotepid
#define SET_REMOTE_NODE(up, val)	(up)->u_remotenode = (val)
#define GET_REMOTE_NODE(up)		(up)->u_remotenode


/*
 * function prototypes for task open file table routines 
 */
void clear_fdbit(int, struct ufile_state *);
boolean_t set_fdbit(int, struct ufile_state *);
int ufalloc(long, int *, struct ufile_entry **, struct ufile_state *);
int falloc(struct file **, int *,  struct ufile_entry **);
int getf(struct file **, long, char *, struct ufile_state *);
int getf_ref_always(struct file **, long, char *, struct ufile_state *);
struct ufile_entry * get_remote_task_fe(int, struct ufile_state *);
void u_set_fe(int, struct ufile_entry *, struct file *, struct ufile_state *); 
void u_set_fe_with_flag(int, struct ufile_entry *, struct file *, 
			struct ufile_state *, int); 
void u_file_table_ref(struct ufile_state *);
void u_file_table_unref(struct ufile_state *);
struct ufile_entry *get_remote_task_fe(int, struct ufile_state *);
void close_task_files(struct ufile_state *);
int getf_by_remote_task(struct file **, long, char *, struct ufile_state *);
void close_task_files_nofree(struct ufile_state *);

/*
 * function prototypes for coredir handling routines
 */
int  add_corepath(char *);
void remove_corepath(void);

/* 
 * locking macros for open file table.
 */
udecl_simple_lock_info(extern ,ufe_lockinfo)

#define U_FE_LOCK_INIT(fe)	\
	usimple_lock_setup(&(fe)->ufe_ofile_lock, ufe_lockinfo)

#define U_FE_LOCK_TERMINATE(fe)	\
	usimple_lock_terminate(&(fe)->ufe_ofile_lock)

struct ufile_entry * enable_fd_slot(int, struct ufile_state *);

/*
 * The U_FILE_STATE_LOCK lock is used when accessing the uf_flags or uf_references
 * fields in the task open file table.  For all other cases the U_FILE_STATE_LOCK_MT
 * is used since we only need to synchronize against threads within a multi-threaded
 * task.
 */
#define U_FILE_STATE_LOCK(ufp) 					\
MACRO_BEGIN							\
{								\
	usimple_lock(&(ufp)->uf_ofile_lock); 			\
}								\
MACRO_END

#define U_FILE_STATE_UNLOCK(ufp) 				\
MACRO_BEGIN							\
{								\
	usimple_unlock(&(ufp)->uf_ofile_lock); 			\
}								\
MACRO_END

#define U_FILE_STATE_LOCK_MT(ufp) 				\
MACRO_BEGIN							\
{								\
	if ((ufp)->utask_need_to_lock)				\
		usimple_lock(&(ufp)->uf_ofile_lock); 		\
}								\
MACRO_END

#define U_FILE_STATE_UNLOCK_MT(ufp) 				\
MACRO_BEGIN							\
{								\
	if ((ufp)->utask_need_to_lock)				\
		usimple_unlock(&(ufp)->uf_ofile_lock); 		\
}								\
MACRO_END

/*
 * The U_FE_LOCK needs to be taken whenever the field ufe_ofile is being 
 * changed from valid to NULL, NULL to  valid, or U_FD_RESERVED to valid.
 * The U_FE_LOCK is also used when looking at another tasks open file 
 * table.  For all other cases, the U_FE_LOCK_MT is used since we only need
 * to synchronize if it is a multi-threaded task.
 */ 
#define U_FE_LOCK(fe) 						\
MACRO_BEGIN							\
{								\
	usimple_lock(&(fe)->ufe_ofile_lock);			\
}								\
MACRO_END

#define U_FE_UNLOCK(fe) 					\
MACRO_BEGIN							\
{								\
	usimple_unlock(&(fe)->ufe_ofile_lock);			\
}								\
MACRO_END

#define U_FE_LOCK_MT(fe,ufp) 					\
MACRO_BEGIN							\
{								\
	if ((ufp)->utask_need_to_lock)				\
		usimple_lock(&(fe)->ufe_ofile_lock);		\
}								\
MACRO_END

#define U_FE_UNLOCK_MT(fe,ufp) 					\
MACRO_BEGIN							\
{								\
	if ((ufp)->utask_need_to_lock)				\
		usimple_unlock(&(fe)->ufe_ofile_lock);		\
}								\
MACRO_END


/*
 * The U_GET_FE macro is used to get a file entry structure pointer for the 
 * current task.  Need to use get_remote_task_fe() if going after another task.
 */
#define U_GET_FE(fd, ufp) 	(((fd) < NOFILE_IN_U) 	? 		\
	((ufp)->uf_entry[(fd)/U_FE_ALLOC_SIZE]  ? 			\
	(ufp)->uf_entry[(fd)/U_FE_ALLOC_SIZE] + 			\
			((fd) %U_FE_ALLOC_SIZE) :			\
	NULL) :								\
	((ufp)->uf_of_entry ?						\
	((((ufp)->uf_of_entry[((fd)-NOFILE_IN_U)/U_FE_OF_ALLOC_SIZE]) +	\
			(((fd)-NOFILE_IN_U) % U_FE_OF_ALLOC_SIZE))) : NULL))


#define u_ar0		np_uthread->uu_ar0
#define u_code		np_uthread->uu_code
#define u_sig		np_uthread->uu_sptr->sig
#define	u_cursig	np_uthread->uu_cursig
#define	u_sflags	np_uthread->uu_sflags
#define	u_tsig		np_uthread->uu_tsig
#define u_share		np_uthread->uu_share
#define u_sptr		np_uthread->uu_sptr
#define u_sigmask	np_uthread->uu_sptr->sigmask
#define uu_sigmask	uu_sptr->sigmask
#define uu_sig		uu_sptr->sig
#define u_nxm_active	np_uthread->uu_sptr->nxm_active
#define u_nxm_ssig	np_uthread->uu_sptr->nxm_ssig
#define u_sigwaitmask	np_uthread->uu_sigwaitmask
#define u_select_event	np_uthread->uu_select_event
#define	u_sigqueue	np_uthread->uu_sigqueue
#define	u_curinfo	np_uthread->uu_curinfo
#define u_tru		np_uthread->uu_tru
#define u_pthid		np_uthread->uu_sptr->pth_id
#define u_shflags	np_uthread->uu_sptr->flags
#define u_cancel_state	np_uthread->uu_sptr->cancel_state
#define u_tsd		np_uthread->uu_tsd

#define u_comm		utask->uu_comm
#define u_calltype	utask->uu_calltype	/* ROMP_DUALCALL */
#define u_logname	utask->uu_logname
#define	u_utnd		utask->uu_utnd
#define	u_file_state	utask->uu_file_state
#define u_fd_newmax	utask->uu_fd_newmax
#define u_task_rq_wait_sum	utask->uu_task_rq_wait_sum
#define uu_cdir		uu_utnd.utnd_cdir
#define uu_rdir		uu_utnd.utnd_rdir

#define u_cdir		utask->uu_cdir
#define u_rdir		utask->uu_rdir
#define u_cred		u_nd.ni_cred
#define	u_ruid		u_procp->p_ruid
#define	u_rgid		u_procp->p_rgid
#define u_uid		u_cred->cr_uid
#define u_gid		u_cred->cr_gid
#define u_ngroups	u_cred->cr_ngroups
#define u_groups	u_cred->cr_groups

#define u_tsize		utask->uu_tsize
#define u_dsize		utask->uu_dsize
#define u_ssize		utask->uu_ssize
#define u_text_start	utask->uu_text_start
#define u_data_start	utask->uu_data_start
#define u_stack_start	utask->uu_stack_start
#define u_stack_end	utask->uu_stack_end
#define u_stack_grows_up utask->uu_stack_grows_up
#ifdef __alpha
#define u_spec		utask->uu_spec
#define u_fbhandler	utask->uu_fbhandler
#endif /* __alpha */

#define u_sigtramp	utask->uu_sigtramp

#define u_lastfile	u_file_state.uf_lastfile
#define	u_ofile_lock	u_file_state.uf_ofile_lock
#define u_handy_lock    utask->uu_handy_lock
#define u_addr_lock	utask->uu_addr_lock
#define u_cmask		utask->uu_cmask

#define u_ru		utask->uu_ru
#define u_cru		utask->uu_cru
#define u_ioch		utask->uu_ioch
#define u_timer		proc->p_timer
#define u_XXX		utask->uu_XXX
#define u_start		utask->uu_start
#define u_acflag	utask->uu_acflag

#define u_maxuprc	utask->uu_maxuprc

#define u_lflags	utask->uu_lflags

#define u_rlimit	utask->uu_rlimit


#define	u_coredir	utask->uu_coredir
#define u_semundo	utask->uu_semundo
#define u_shmsegs	utask->uu_shmsegs

#define u_argp          utask->uu_argp
#define u_envp          utask->uu_envp
#define u_arg_size      utask->uu_arg_size
#define u_env_size      utask->uu_env_size
#define u_compat_mod    utask->uu_compat_mod
#define u_exitp         utask->uu_exitp           /* _LMF_ */
#define u_auditmask     utask->uu_auditmask
#define u_audit_cntl    utask->uu_audit_cntl
#define u_set_uids      utask->uu_set_uids
#define u_mgr           utask->uu_mgr
#define u_handle        utask->uu_handle
#define u_sigresvcnt	utask->uu_sigresvcnt
#define u_sel_cache	utask->uu_sel_cache
#define u_sel_size	utask->uu_sel_size

/*
 * Reserved TSD key definitions
 *
 * The NFS server, the Streams subsystem, and CFS have reserved keys
 * with which to access per-subsystem thread-specific data in the kernel.
 * All other subsystems must explicitly reserve a TSD key via the
 * interface.  (See thread.h)
 */
#define TSD_NFS_KEY             0
#define TSD_STREAMS_KEY         1
#define TSD_CFS_KEY		2

#define TSD_FIRST_AVAIL_KEY     3

/*
 * Convenience macros for the NFS server to short-circuit the TSD
 * access methods
 */
#define NFS_SERVER_TSD		u.u_tsd[TSD_NFS_KEY]

#define NFS_SERVER_TSD_BY_THD(__th__)				\
    thread_to_np_uthread(__th__)->uu_tsd[TSD_NFS_KEY]

/*
 * Convenience macro for CFS to short-circuit the TSD
 * access methods
 */
#define CFS_SENT_INFO		((struct cfs_sent_info *)u.u_tsd[TSD_CFS_KEY])

/*
 * Per-utask locks for MP synchronization.
 */

/*
 * This lock covers miscellaneous Unix-related per-task data:
 *   u.u_ru, u.u_cru, u.logname, u.u_acflag, u.u_rdir, u.u_cmask,
 *   and all members of u.u_rlimit except RLIMIT_CPU.
 */
#define U_HANDY_LOCK() 						\
MACRO_BEGIN							\
{								\
	register struct utask *utask;				\
	utask = u.utask;					\
	if (utask->uu_file_state.utask_need_to_lock)		\
		usimple_lock(&utask->uu_handy_lock);		\
}								\
MACRO_END
#define U_HANDY_UNLOCK() 					\
MACRO_BEGIN							\
{								\
	register struct utask *utask;				\
	utask = u.utask;					\
	if (utask->uu_file_state.utask_need_to_lock)		\
		usimple_unlock(&utask->uu_handy_lock);		\
}								\
MACRO_END

/*
 * multithreaded synchronization to Unix address space parameters:
 *   u.u_curbrk, u.u_minbrk, u.u_lflags, u.u_stack_start,
 *   u.u_stack_end, process sigactions, and POSIX timers
 */
#define U_ADDR_LOCK() 						\
MACRO_BEGIN							\
{								\
	register struct utask *utask;				\
	utask = u.utask;					\
	if (utask->uu_file_state.utask_need_to_lock)		\
		lock_write(&utask->uu_addr_lock);		\
}								\
MACRO_END

#define U_ADDR_UNLOCK() 					\
MACRO_BEGIN							\
{								\
	register struct utask *utask;				\
	utask = u.utask;					\
	if (utask->uu_file_state.utask_need_to_lock)		\
		lock_done(&utask->uu_addr_lock);		\
}								\
MACRO_END

#define U_ADDR_LOCK_TRY()					\
	(u.utask->uu_file_state.utask_need_to_lock ?		\
		lock_try_write(&u.utask->uu_addr_lock) : 1)

/*
 * Unconditional versions of the U_ADDR_LOCK family
 */
#define U_ADDR_LOCK_ALWAYS()					\
MACRO_BEGIN							\
	lock_write(&u.utask->uu_addr_lock);			\
MACRO_END

#define U_ADDR_UNLOCK_ALWAYS()					\
MACRO_BEGIN							\
	lock_done(&u.utask->uu_addr_lock);			\
MACRO_END

/*
 * Unconditional version of U_ADDR_LOCK that targets a proc
 */
#define U_ADDR_LOCK_ALWAYS_PROC(__p__)					\
MACRO_BEGIN								\
	struct utask *utaskp = proc_to_utask(__p__);			\
	lock_write(&utaskp->uu_addr_lock);				\
MACRO_END

#define U_ADDR_UNLOCK_ALWAYS_PROC(__p__)				\
MACRO_BEGIN								\
	struct utask *utaskp = proc_to_utask(__p__);			\
	lock_done(&utaskp->uu_addr_lock);				\
MACRO_END

#endif	/* _KERNEL */

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif	/* _SYS_USER_H_ */
