/*
 * *****************************************************************
 * *                                                               *
 * *   Copyright 2002 Compaq Information Technologies Group, L.P.  *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*	
 *	@(#)$RCSfile: tcp_var.h,v $ $Revision: 4.2.92.4 $ (DEC) $Date: 2002/05/31 16:19:08 $
 */ 
/*
 */
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * OSF/1 Release 1.0
 */
/*
 * Copyright (C) 1988,1989 Encore Computer Corporation.  All Rights Reserved
 *
 * Property of Encore Computer Corporation.
 * This software is made available solely pursuant to the terms of
 * a software license agreement which governs its use. Unauthorized
 * duplication, distribution or sale are strictly prohibited.
 *
 */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	Base:	tcp_var.h	7.8 (Berkeley) 6/29/88
 *	Merged:	tcp_var.h	7.10 (Berkeley) 6/28/90
 */

#ifndef _TCP_VAR_H_
#define _TCP_VAR_H_
/*
 * Kernel variables for tcp.
 */

/*
 * Tcp control block, one per tcp; fields:
 */

struct tcpcb {
#ifdef __arch64__
	struct	ovtcphdr *seg_next;	/* sequencing queue */
	struct	ovtcphdr *seg_prev;
#else  /* not __arch64__ */
	struct	tcpiphdr *seg_next;	/* sequencing queue */
	struct	tcpiphdr *seg_prev;
#endif /* __arch64__ */
	short	t_dupacks;		/* consecutive dup acks recd */
	short	t_softerror;		/* possible error not yet reported */
	short	t_rxtshift;		/* log(2) of rexmt exp. backoff */
	short	t_rxtcur;		/* current retransmit value */
	u_short	t_flags;
#define	TF_ACKNOW	0x01		/* ack peer immediately */
#define	TF_DELACK	0x02		/* ack, but try to delay it */
#define	TF_NODELAY	0x04		/* don't delay packets to coalesce */
#define	TF_NOOPT	0x08		/* don't use tcp options */
#define	TF_SENTFIN	0x10		/* have sent FIN */
#define TF_REQ_SCALE    0x0020		/* have/will request window scaling */
#define TF_RCVD_SCALE   0x0040		/* other side has requested scaling */
#define TF_UIOMOVED     0x0080		/* UIOMOVE mbufs in use (DART) */
	u_short t_flags2;		/* more flags */
#define TF2_PUSH	0x0001		/* set push for outbound data packets */
#define TF2_NODELACK	0x0002		/* set no delayed ack for incoming packets*/
#define TF2_TSOPTENA	0x0004		/* enable (rfc1323)request of timestamp option */
#define TF2_TSOPT	0x0008		/* connection is running with timestamp enabled */
#define TF2_PAWS	0x0010		/* enable (rfc1323)PAWS option */
#define TF2_CLUA_ALIAS  0x0020		/* this is a cluster alias connection */
#define TF2_CLUA_CNXQ   0x0040		/* a cluster alias connection is queued for registration */
#define TF2_CLUA_CNXR   0x0080		/* a cluster alias connection is registered */
#define TF2_SACKENA	0x0100		/* enable PAWS option(rfc1323) */
#define TF2_SACK	0x0200		/* enable SACK option(rfc2018) */
#define TF2_KEEP_PANIC	0x0400		/* when tcp_keep_panic flag set will
					   panic when keep_cnt exceeded */
#define TF2_DELAY_RXMT	0x0800		/* used to delay rexmit timeout for ics channel */
#define TF2_INHERIT	(TF2_PAWS|TF2_TSOPTENA|TF2_SACKENA)
#ifndef  __VMS
	int pad_a[9];
#else /* __VMS */
	int pad_a[11];
#endif /* __VMS */
/* 64-byte alignment */
	struct	inpcb *t_inpcb;		/* back pointer to internet pcb */
	unsigned int t_lastaction;	/* start of inactivity period */
#define t_TSRecent_age t_lastaction	/* prevents discard of valid packets when tstamp wraps */
	tcp_seq	snd_wnd;		/* send window */
	tcp_seq	snd_cwnd;		/* congestion-controlled window */
	tcp_seq	snd_max;		/* highest sequence number sent */
	tcp_seq	snd_una;		/* send unacknowledged */
	tcp_seq	snd_nxt;		/* send next */
	tcp_seq	snd_up;			/* send urgent pointer */
	tcp_seq	snd_wl1;		/* window update seg seq number */
	tcp_seq	snd_wl2;		/* window update seg ack number */
	tcp_seq	iss;			/* initial send sequence number */
	tcp_seq	max_sndwnd;		/* largest window peer has offered */
	tcp_seq snd_ssthresh;		/* snd_cwnd size threshhold */
	u_char  snd_scale;		/* window scaling for send window */
	u_char  request_r_scale;	/* pending window scaling */
	u_char  requested_s_scale;
	char pad_b[1];
#ifndef  __VMS
	int pad_c[1];
#else /* __VMS */
	int pad_c[2];
#endif /* __VMS */
/* 64-byte alignment */
	tcp_seq	rcv_wnd;		/* receive window */
	tcp_seq	rcv_adv;		/* advertised window */
	tcp_seq	rcv_nxt;		/* receive next */
	tcp_seq	rcv_up;			/* receive urgent pointer */
	tcp_seq max_rcvd;		/* most peer has sent into window */
	tcp_seq	irs;			/* initial receive sequence number */
	u_short	t_maxseg;		/* maximum segment size */
	u_char  rcv_scale;		/* window scaling for recv window */
	char pad_i[1];
  	unsigned int t_TSRecent ;	/* timestamp of most recent rcvd packet */
  	unsigned int t_TSEcr ;		/* peers timestamp sent as the TSEcr field of tsopt */
	tcp_seq	 Last_ACK_sent;		/* last ack to determine if the window moves when updating t_TSEcr */
	struct tcp_sack_field *t_sack;	/* pointer to last sack field received */
#ifndef  __VMS
	int pad_d[4];
#else /* __VMS */
	int pad_d[5];
#endif /* __VMS */
/* 64-byte alignment */
	tcp_seq	t_rtseq;		/* sequence number being timed */
	short	t_srtt;			/* smoothed round-trip time */
	short	t_rttvar;		/* variance in round-trip time */
	u_short	t_rttmin;		/* minimum rtt allowed */
	u_short	t_msslimit;		/* user-specified max segment size */
	int	t_rptr2rxt;		/* Repeat counter for R2 RXT timer */
	int	t_rptr2cur;		/* Current repeat counter for R2 timer */
	char	t_oobflags;		/* have out-of-band data */
	char	t_iobc;			/* input out-of-band character */
#define	TCPOOB_HAVEDATA	0x01
#define	TCPOOB_HADDATA	0x02
	char	t_force;		/* 1 if forcing out a byte */
	char pad_e[1];
	u_short	t_peermss;		/* max segment size offered by peer */
	short	t_persistcntr;		/* persist timeout for aborting */
	short	t_keepidle;		/* idle time before keepalive probe */
	short	t_keepintvl;		/* interval between keepalive probes */
	short	t_keepcnt;		/* number of keepalive probes */
	short	t_keepinit;		/* initial connect timeout */
	short	t_state;		/* state of this connection */
	short pad_f[1];
#ifndef  __VMS
	long pad_g[3];
#else /* __VMS */
	long pad_g[6];
#endif /* __VMS */
/* 64-byte alignment */
	unsigned int t_startrtt;	/* starting round trip time */
	tcp_seq	t_timestamp;		/* used by slowtimo */
	unsigned int t_timer[TCPT_NTIMERS];	/* tcp timers */
	int	clua_seq_wrap_guard;	/* cluster alias sequence wrap guard */
	int pad_h[1];
};

#define MAX_SACK_ENTRIES 32
/*
 * the tcp_sack_field is used to save a sack update
 * recieved as an option in incoming packet once
 * tcp_outputs the missing packet then the number
 * of entries is set to 0
 */
struct tcp_sack_field {
	short number;		/* the number of entries in this sack update */
	short sack_index;
	struct tcp_sack_entry {
		int ledge ;	/* Left edge of the segment */
		int redge ;	/* right edge of the segment */
	} entries[MAX_SACK_ENTRIES] ;
} ;

#define	intotcpcb(ip)	((struct tcpcb *)(ip)->inp_ppcb)
#define	sototcpcb(so)	(intotcpcb(sotoinpcb(so)))

/*
 * The smoothed round-trip time and estimated variance
 * are stored as fixed point numbers scaled by the values below.
 * For convenience, these scales are also used in smoothing the average
 * (smoothed = (1/scale)sample + ((scale-1)/scale)smoothed).
 * With these scales, srtt has 3 bits to the right of the binary point,
 * and thus an "ALPHA" of 0.875.  rttvar has 2 bits to the right of the
 * binary point, and is smoothed with an ALPHA of 0.75.
 */
#define	TCP_RTT_SCALE		8	/* multiplier for srtt; 3 bits frac. */
#define	TCP_RTT_SHIFT		3	/* shift for srtt; 3 bits frac. */
#define	TCP_RTTVAR_SCALE	4	/* multiplier for rttvar; 2 bits */
#define	TCP_RTTVAR_SHIFT	2	/* multiplier for rttvar; 2 bits */

/*
 * The initial retransmission should happen at rtt + 4 * rttvar.
 * Because of the way we do the smoothing, srtt and rttvar
 * will each average +1/2 tick of bias.  When we compute
 * the retransmit timer, we want 1/2 tick of rounding and
 * 1 extra tick because of +-1/2 tick uncertainty in the
 * firing of the timer.  The bias will give us exactly the
 * 1.5 tick we need.  But, because the bias is
 * statistical, we have to test that we don't drop below
 * the minimum feasible timer (which is 2 ticks).
 * This macro assumes that the value of TCP_RTTVAR_SCALE
 * is the same as the multiplier for rttvar.
 */
#define	TCP_REXMTVAL(tp) \
	(((tp)->t_srtt >> TCP_RTT_SHIFT) + (tp)->t_rttvar)

#ifdef __arch64__
#define REASS_MBUF(ov) (*(struct mbuf **)&((struct ovtcphdr *)(ov))->ti_rmb)
#else
#define REASS_MBUF(ti) (*(struct mbuf **)&((ti)->ti_t))
#endif /* __arch64__ */

#include <netinet/ip_stat.h>

#define _TCBHASHSIZE	512		/* default value for tcbhashsize */
#define TCP_SENDSPACE	60*1024		/* default value for tcp_sendspace */
#define TCP_RECVSPACE	60*1024		/* default value for tcp_recvspace */
#define TCPREXMTTHRESH	3		/* default retransmit threshold */

#ifdef _KERNEL

#ifndef REPLICATED
#define REPLICATED
#endif

#if	NETSYNC_LOCK
extern	simple_lock_data_t	misc_tcp_lock;
#define TCPMISC_LOCKINIT()	simple_lock_setup(&misc_tcp_lock, misc_tcp_li)
#define TCPMISC_LOCK()		simple_lock(&misc_tcp_lock)
#define TCPMISC_UNLOCK()	simple_unlock(&misc_tcp_lock)
#else	/* !NETSYNC_LOCK */
#define TCPMISC_LOCKINIT()
#define TCPMISC_LOCK()
#define TCPMISC_UNLOCK()
#endif

#define TCBMAXQUEUES 64
extern	REPLICATED int ipqshiftbits;
#define TCBQINDEX(addr) ((((addr)>>ipqshiftbits)&0xff) % tcbqNum)
extern	REPLICATED int tcp_skip_local_cwnd;
extern	REPLICATED int tcp_cwnd_segments;
extern	REPLICATED int tcp_compat_42;
extern	REPLICATED int tcp_urgent_42;
extern	REPLICATED int tcp_sack_enabled;
extern	REPLICATED int tcp_tsopt_enabled;
extern	REPLICATED int tcp_paws_enabled;

extern	REPLICATED int tcbqNum;
extern	struct	inpcb tcb[];		/* head of queue of active tcpcb's */
extern	struct	tcpstat tcpstat;	/* tcp statistics */
extern	struct	inpcb tcbinit;		/* listeners and some not-yet-connected pcbs */
extern	struct	__inpcb_hashbucket *tcbhashtab; /* connected pcbs */
extern	void * REPLICATED replicated_tcbhbiss;
#define	tcbhbiss ((tcp_seq*)replicated_tcbhbiss)/* modifier for iss on a per hb basis */
extern	REPLICATED int tcbhashmod;	/* buckets in tcbhashtab */
extern	REPLICATED int tcbhashsize;	/* initial value for tcbhashmod */
extern	int tcbquickbindtab[] ;
extern struct tcb_hash_stats_struct {
	int     ChainHiwatCount ;
	int     ChainHiwatThreshold ;
	int     EmptyWarnCount ;
	int     EmptyWarnThreshold ;
	int     FastActive ;
	int     FastIdle ;
	int     SlowActive ;
	int 	AllTimeSlowActive ;
	int     SlowIdle ;
	int     ChainHiwat ;
	int 	AllTimeChainHiwat ;
	int     ChainLowat ;
	int     EmptyBuckets ;
	int     TotalBuckets ;
	int     TotalTcpConns ;
	int     AllTimeMaxTcpConns ;
} tcbhashstats ;

extern struct	__inpcb_hashbucket *HashHiwatBucket ;
extern struct	__inpcb_hashbucket *HashLowatBucket ;

#define FAILSAFEHASHTABLE	32
extern int total_tcp_inpcbs;
extern REPLICATED int tcp_keepalive_default;	/* used to make keep alive the default mod */
extern REPLICATED int tcp_free_delay;		/* delay in slowtimo ticks to free tcp inps */

struct tcpquantumentry {
	int quantum;
	int index;
	struct inpcb *inp;
};
/*
 * contains control information needed for each rad in system.
 */
extern struct tcpradcontrol {
	struct tcpradcontrol	   *trc_next;
	struct   __inpcb_hashbucket trc_tcbinit;
	unsigned int		   *trc_tcbquicklistentab;
	int			    trc_fastserver;
#if	NETSYNC_LOCK
	simple_lock_data_t 	    trc_fastlock;
#endif
	struct inpcb		   *trc_fasttimo_next;
#if	NETISR_THREAD
	thread_t		    trc_fastthread;
#endif
	int			    trc_slowserver;
#if	NETSYNC_LOCK
	simple_lock_data_t 	    trc_slowlock;
#endif
	struct inpcb		   *trc_slowtimo_next;
#if	NETISR_THREAD
	thread_t		    trc_slowthread;
#endif
	struct tcpquantumentry	   *trc_tcpslowquantum;
	int			    trc_maxperiod;
#if	NETISR_THREAD
	wait_queue_data_t	    trc_fast_wait;
	wait_queue_data_t	    trc_slow_wait;
#endif
	struct inpcb		   *trc_fast_active;
	struct inpcb		   *trc_slow_active;
	int			    trc_lperiod;
} *tcpradcontrollist ;

#define TCPRADCONTROL ((struct tcpradcontrol*)current_rad()->tcpradcontrol)
#define RAD2TCPRADCONTROL(__rad) ((struct tcpradcontrol*)((__rad)->tcpradcontrol))
#define RADID2TCPRADCONTROL(__radid) ((struct tcpradcontrol*)(rad_id_to_rad(__radid)->tcpradcontrol))
/* Enqueue the inp onto the fasttimo list.  The atomic test and set
 * allows the timer thread and the signalling thread to coordinate
 * the enqueue and the dequeue of the inp on the worklist.  The
 * simple lock is used to protect the integrity of the list */
#define SIGNAL_TCPFASTTIMO(_tp, _inp) \
(_tp)->t_flags |= TF_DELACK; \
if(test_and_set_l(1,&(_inp)->inp_slowref)) { \
	simple_lock(&RAD2TCPRADCONTROL(_inp->inp_rad)->trc_fastlock); \
	if((_inp)->inp_fasttimo_next == NULL) { \
		(_inp)->inp_fasttimo_next = RAD2TCPRADCONTROL(_inp->inp_rad)->trc_fasttimo_next; \
		INPCBRC_REF((_inp)); \
		RAD2TCPRADCONTROL(_inp->inp_rad)->trc_fasttimo_next = (_inp); \
	} \
	simple_unlock(&RAD2TCPRADCONTROL(_inp->inp_rad)->trc_fastlock); \
}

/* Enqueue the inp onto the slowtimo list.  The atomic test and set
 * allows the timer thread and the signalling thread to coordinate
 * the enqueue and the dequeue of the inp on the worklist.  The
 * simple lock is used to protect the integrity of the list */
#define SIGNAL_TCPSLOWTIMO(_tp, _inp) \
if(test_and_set_l(0,&_inp->inp_slowref)) {\
	simple_lock(&RAD2TCPRADCONTROL(_inp->inp_rad)->trc_slowlock); \
	_inp->inp_slowtimo_next = RAD2TCPRADCONTROL(_inp->inp_rad)->trc_slowtimo_next; \
	RAD2TCPRADCONTROL(_inp->inp_rad)->trc_slowtimo_next = _inp; \
	INPCBRC_REF(_inp); \
	simple_unlock(&RAD2TCPRADCONTROL(_inp->inp_rad)->trc_slowlock); \
}

#endif /* _KERNEL */
#endif /* _TCP_VAR_H_*/
