/*
 * *****************************************************************
 * *                                                               *
 * *   Copyright 2002 Compaq Information Technologies Group, L.P.  *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: vm_umap.h,v $ $Revision: 1.1.46.1 $ (DEC) $Date: 2002/10/08 18:18:28 $
 */
#ifndef	__VM_UMAP__
#define	__VM_UMAP__ 1

#ifdef __cplusplus
extern "C" {
#endif

#include <kern/lock.h>
#include <vm/vm_vlock.h>
#include <sys/mman.h>

/*
 * user mode private map structure
 * The following members can change after creation, and are protected
 * by the map lock:
 * 
 * 	um_actual_size
 * 	um_lock_future
 * 	um_unload_all
 * 	um_vlock
 */

struct u_map_private {
	vm_size_t		um_maxvas;	/* Maximum VAS allowed */
	vm_size_t		um_maxrss;	/* Maximum rss allowed */
	vm_size_t		um_rss;		/* Resident pages in map */
	vm_size_t		um_actual_size;	/* actual size */
	unsigned long		um_flags;
	struct vm_vlock		*um_vlock;	/* Virtual space lock by K */
};
typedef	struct u_map_private	*u_map_private_t;
/*
 * Bit masks for um_flags field
 */
#define UM_LOCK_FUTURE_READ     0x1             /* lock future R pages */
#define UM_LOCK_FUTURE_WRITE    0x2             /* lock future W pages */
#define UM_LOCK_FUTURE_EXEC     0x4             /* lock future E pages */
#define UM_LOCK_FUTURE_ALL      0x8             /* lock all future pages */
#define UM_UNLOAD_ALL           0x100           /* unloading all address sp */

/*
 * The code that makes use of these bits assume
 * that the user-level API bit definitions match
 * those defined in the u_map_private structure.
 * The follow conditionals verify that the
 * definitions are consistent.
 */

#if UM_LOCK_FUTURE_READ != PROT_READ
#error	"UM_LOCK_FUTURE_READ != PROT_READ, check definitions"
#endif	/* UM_LOCK_FUTURE_READ != PROT_READ */

#if UM_LOCK_FUTURE_WRITE != PROT_WRITE
#error	"UM_LOCK_FUTURE_WRITE != PROT_WRITE, check definitions"
#endif	/* UM_LOCK_FUTURE_WRITE != PROT_WRITE */

#if UM_LOCK_FUTURE_EXEC != PROT_EXEC
#error	"UM_LOCK_FUTURE_EXEC != PROT_EXEC, check definitions"
#endif	/* UM_LOCK_FUTURE_EXEC != PROT_EXEC */

#define UM_LOCK_FUTURE          (UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_WRITE \
                                | UM_LOCK_FUTURE_EXEC)
#define UM_LOCK_FUTURE_TEXT     (UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_EXEC)
#define UM_LOCK_FUTURE_DATA     (UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_WRITE)

extern vm_size_t u_map_actual_size(vm_map_t);

#define	vm_mape_lockaddr(VME) simple_lock_addr((VME)->vme_faultlock)

#define OBJECT_START_VA(ep) ((unsigned long)(ep->vme_start - (long)ep->vme_offset))
#define OBJECT_END_VA(ep) ((unsigned long)(OBJECT_START_VA(ep) + ep->vme_object->ob_size))

#define VA_TO_OFFSET(ep,va) (va - OBJECT_START_VA(ep))
#define OFFSET_TO_VA(ep,off) (off + OBJECT_START_VA(ep))

#define VA_ABOVE_OBJECT(ep,va) ((unsigned long)va >= OBJECT_END_VA(ep)) 
#define VA_BELOW_OBJECT(ep,va) ((unsigned long)va < OBJECT_START_VA(ep))
#define VA_TOUCHES_OBJECT(ep,va) (!VA_ABOVE_OBJECT(ep, va) && !VA_BELOW_OBJECT(ep, va))

#define VA_ABOVE_MAPE(ep,va) ((unsigned long)va >= ep->vme_end)
#define VA_BELOW_MAPE(ep,va) ((unsigned long)va < ep->vme_start)
#define VA_TOUCHES_MAPE(ep,va) (!VA_ABOVE_MAPE(ep, va) && !VA_BELOW_MAPE(ep, va))

/*
 * The vm_mape_faultwait is called by a thread that has the address map
 * read or write locked.  The thread goes to sleep on the fault lock until
 * vme_faults count goes to zero.
 *
 * If the object type is ANON and it has a ref count of more than one then
 * it may mean that we have multiple map entries mapping the object.  In this
 * case it is necessary to wait on each of their fault counts.  Of course,
 * the greater than one ref count could simply mean that there is more than
 * one process using the object.  In this case it is not necessary to parse
 * through all the map entries to find ones to wait on, which can be expensive.
 *
 * The code does the following:
 * If we are ANON and have a ref count > 1
 * 	1) Check if the current map entry covers the entire object.  If so,
 * 	   we don't need to do the other stuff, so simply check this map
 * 	   entry and get out.
 * 	2) Otherwise, starting at the first map entry in the map, search for
 * 	   ones that belong to the target object.  On the first one, save the
 * 	   last possible VA covered by the object, this will allow early exit.
 * 	   Now, for every map entry mapping a portion of the object, wait for
 * 	   it's fault count to go to 0.  If we reach the point where the next
 * 	   map entr's starting VA is outside the calculated endva then we are
 * 	   done.  
 *
 * NOTE:
 * 	The worse case scenario is we have to traverse all the map entries.
 */
#define vm_mape_faultwait(VME) {					\
	vm_map_t	map = (VME)->vme_map;				\
	vm_object_t	obj = (VME)->vme_object;			\
	int		refcnt = obj->ob_ref_count;			\
	if (obj->ob_type == OT_ANON && refcnt > 1) {			\
		if (obj->ob_size <= ((VME)->vme_end - (VME)->vme_start)) { \
			vm_faultwait(VME);				\
		}							\
		else {							\
			vm_map_entry_t tmpvme = vm_map_first_entry(map);\
			vm_offset_t endva = 0L;				\
			while (tmpvme != (vm_map_entry_t)map) {		\
				if (tmpvme->vme_object == obj) {	\
					if (!endva)			\
					    endva = tmpvme->vme_start + \
						    obj->ob_size;	\
					vm_faultwait(tmpvme);		\
					if (tmpvme->vme_end > endva)	\
						break;			\
				}					\
				tmpvme = tmpvme->vme_next;		\
			}						\
		}							\
	}								\
	else								\
	    vm_faultwait(VME);						\
}

#define vm_faultwait(VME) {						\
	if ((VME)->vme_faults) {					\
		simple_lock(&(VME)->vme_faultlock);			\
		if ((VME)->vme_faults) {				\
			do {						\
				(VME)->vme_faultwait = 1;		\
				assert_wait_mesg((vm_offset_t)(VME),	\
						  FALSE, "fault");	\
				simple_unlock(&(VME)->vme_faultlock);	\
				thread_block();				\
				simple_lock(&(VME)->vme_faultlock);	\
			} while ((VME)->vme_faults);			\
		}							\
		simple_unlock(&(VME)->vme_faultlock);			\
	}								\
}

/*
 * Described return states from u_map_is_wired, see function for definitions.
 */
typedef enum { UMAP_NOCHG, UMAP_KWIRED, UMAP_UWIRED, UMAP_NOWIRED } u_mapwire_t;

#ifdef	_KERNEL
#if defined(_USE_KERNEL_PROTOS)
/*
 * forward declaration for ANSI & C++
 */
struct vnode;

extern boolean_t
	lock_future(vm_map_entry_t, vm_map_t),
	u_map_entry_delete_check(struct vm_map *, vm_map_entry_t, 
			vm_offset_t, vm_offset_t),
	u_map_entry_clip_check(struct vm_map *, vm_map_entry_t,
			vm_offset_t, vm_offset_t),
	u_map_check_protection(struct vm_map *, vm_offset_t,
			vm_offset_t, vm_prot_t),
	u_map_is_locked(struct vm_map *, vm_offset_t, vm_offset_t);

u_mapwire_t
	u_map_is_wired(vm_map_entry_t, vm_offset_t, vm_offset_t, vm_prot_t);

extern void
	u_map_grow_actual(struct vm_map *, vm_size_t),
	u_map_free_vas(struct vm_map *, vm_size_t, vm_size_t),
	u_map_init(void);

extern int
	u_dev_create(struct vm_map *, struct vnode *, vm_offset_t),
	u_map_copy_overwrite(struct vm_map *, vm_offset_t,
			struct vm_map_copy *, boolean_t, vm_size_t),
	u_map_copyin(struct vm_map *, vm_offset_t, vm_size_t,
			boolean_t, struct vm_map_copy **),
	u_map_copyout(struct vm_map *, vm_offset_t *, struct vm_map_copy *),
	u_map_entry_split(struct vm_map_entry *, vm_offset_t, vm_size_t);

extern kern_return_t
	u_map_allocate(struct vm_map *, struct vm_object *, vm_offset_t,
			vm_offset_t *, vm_size_t, boolean_t),
	u_map_control(struct vm_map *, vm_offset_t, vm_offset_t,
			vm_control_t, int),
	u_map_create(struct vm_map *),
	u_map_fault(struct vm_map *, vm_offset_t, vm_prot_t, vm_fault_t),
	u_map_grow_vas(struct vm_map *, vm_size_t, vm_size_t),
	u_map_protect(struct vm_map *, vm_offset_t, vm_offset_t,
			vm_prot_t, boolean_t),
	u_map_delete(struct vm_map *, vm_offset_t, vm_offset_t, boolean_t),
	u_map_enter(struct vm_map *, vm_offset_t *, vm_size_t, vm_offset_t,
			boolean_t, struct vm_object *, vm_offset_t, boolean_t,
			vm_prot_t, vm_prot_t, vm_inherit_t),
	u_map_grow(struct vm_map *, struct vm_object *, vm_offset_t,
			vm_offset_t *, vm_size_t, boolean_t, boolean_t),
	u_map_lockvas(struct vm_map *, vm_offset_t, vm_offset_t, int);

#endif /* defined(_USE_KERNEL_PROTOS) */
#endif	/* _KERNEL */

#ifdef __cplusplus
}
#endif
#endif /* !__VM_UMAP__ */
