/*
 * *****************************************************************
 * *                                                               *
 * *   Copyright 2002 Compaq Information Technologies Group, L.P.  *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
#pragma ident "@(#)$RCSfile: vfs_conf.c,v $ $Revision: 4.2.124.1 $ (DEC) $Date: 2001/10/16 13:00:44 $"
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * OSF/1 Release 1.0
 */
/*
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)vfs_conf.c	3.2 (Berkeley) 5/30/91
 */

#include <sys/param.h>
#include <sys/mount.h>
#include <sys/user.h>
#include <sys/vnode.h>
#include <mach/vm_param.h>
#include <sys/sysconfig.h>
#include <sys/errno.h>
#include <sys/lock_types.h>
#include <sys/malloc.h>
#include <vm/vm_ubc.h>

/*
 * These define the root filesystem and device.
 */
struct vnode *rootvp = (struct vnode *)0;
struct mount *rootfs = (struct mount *)0;
struct vnode *rootdir = (struct vnode *)0;
char *rootmpname = (char *) NULL;
static const char srootmpname[] = "/";
const char default_rootname[] = "root_device";

/*
 * These define the boot filesystem and device,
 * which are used by cluster file system.
 */
struct vnode *bootvp = (struct vnode *)0;
struct mount *bootfs = (struct mount *)0;
struct vnode *bootdir = (struct vnode *)0;
char *bootmpname = (char *) NULL;
const char default_bootname[] = "local_root";

/*
 * Set up the filesystem operations for vnodes.
 * The types are defined in mount.h.
 */
extern	struct vfsops ufs_vfsops;

/*
 * Do metadata only in presto
 */
long prmetaonly = 0;

#include "presto.h"
#if NPRESTO == 0
unsigned long prflags;
#endif /* NPRESTO */

#include <nfs.h>
#include <nfs_server.h>
#include <rpc.h>
#if	NFS && !NFS_DYNAMIC
extern	struct vfsops nfs_vfsops;
extern	struct vfsops nfs3_vfsops;
#endif
#if	!RPC && !NFS && !NFS_SERVER
	/*
	 * If NFS is not configured then the rpc subsystem will not be
	 * linked into the kernel, thereby leaving the routine rpcinit
	 * undefined.  This stub routine will take its place if NFS is 
	 * not configured.  Placed here beacuse this file is built on
	 * a per platform basis.
	 */
void 
rpcinit(void) {}

/*
 * Hack to fix SAS build in MUP.  Real klm_lockctl is in 
 * kernel/klm/klm_lockmgr.c
 */
int 
klm_lockctl(
	char *lh,
	char *ld,
	int cmd,
	char *cred,
	int clid,
	int whereto)
{
	return(EOPNOTSUPP);
}
#endif  /* RPC */

#include <cdfs.h>
#if	CDFS && !CDFS_DYNAMIC
extern  struct vfsops cdfs_vfsops;
#endif
#if CDFS == 0
struct vnodeops cdfs_vnodeops;
#endif

#include <dvdfs.h>
#if	DVDFS && !DVDFS_DYNAMIC
extern  struct vfsops dvdfs_vfsops;
#endif
#if DVDFS == 0
struct vnodeops dvdfs_vnodeops;
#endif

#include <msfs.h>

#if	MSFS == 0
int	msfs_mntbusybuf() {return(0);}
void	msfs_mntflushbuf() {return;}
int     advfs_freezefs() {return(0);}
struct vfs_ubcops msfs_ubcops;
struct vnodeops msfs_vnodeops;
#endif

#if     MSFS && !MSFS_DYNAMIC
extern  struct vfsops msfs_vfsops;
#endif

#include <dcedfs.h>
#if defined(DCEDFS) && DCEDFS
extern  struct vfsops dfs_vfsops;
#endif

#include <dceefs.h>
#if defined(DCEEFS) && DCEEFS
extern  struct vfsops efs_vfsops;
#endif

#include <ffm_fs.h>
#if FFM_FS && !FFM_FS_DYNAMIC
extern struct vfsops ffm_vfsops;
#endif
#if FFM_FS == 0
int
ffm_inactive(void) {return(0);}
#endif

extern  struct vfsops procfs_vfsops;

extern	struct vfsops fdfs_vfsops;

/*
 * Cluster related vfsop vectors.
 */
#include <cfs.h>
extern	struct vfsops cfs_vfsops;

/* #include <cspecfs.h> */
extern	struct vfsops cspecvfsops;

#include <autofs.h>
extern struct vfsops autofs_vfsops;

/********************** Note *********************
 *
 * When adding any file system type past MOUNT_MAXTYPE (sys/mount.h)
 * the generic file system ADDON should be used. Meaning,
 * the vnode v_tag field should be set to VT_ADDON, and file
 * system types should return MOUNT_ADDON. This ensures user
 * level utilities will recognize additional file systems as the
 * generic type "addon".
 *
 * The subordinate file system user level mount command
 * (/sbin/mount_xxx) will need to specify the index into the
 * vfssw structure to pass to mount(2).
 *
 ********************** End Note *****************/
struct vfsops_with_fsname vfssw[] = {
        (struct vfsops *)0,	"unknown",	/* 0 = MOUNT_NONE */
	&ufs_vfsops,		"ufs",		/* 1 = MOUNT_UFS */
#if	NFS && !NFS_DYNAMIC
	&nfs_vfsops,		"nfs",		/* 2 = MOUNT_NFS */
#else
	(struct vfsops *)0,	"",		/* 2 = MOUNT_NFS */
#endif
	(struct vfsops *)0,	"",		/* 3 = MOUNT_MFS */
	(struct vfsops *)0,	"",		/* 4 = MOUNT_PC */
	(struct vfsops *)0,    	"",		/* 5 = MOUNT_S5FS */
#if	CDFS && !CDFS_DYNAMIC
        &cdfs_vfsops,           "cdfs",		/* 6 = MOUNT_CDFS */
#else
	(struct vfsops *)0,    	"",		/* 6 = MOUNT_CDFS */
#endif
#if defined(DCEDFS) && DCEDFS
	&dfs_vfsops,            "dfs",		/* 7 = MOUNT_DFS */
#else
	(struct vfsops *)0,     "",		/* 7 = MOUNT_DFS */
#endif
#if defined(DCEEFS) && DCEEFS
	&efs_vfsops,            "efs",		/* 8 = MOUNT_EPI */
#else
	(struct vfsops *)0,     "",		/* 8 = MOUNT_EPI */	
#endif  
	&procfs_vfsops,		"procfs",	/* 9 = MOUNT_PROCFS */
#if     MSFS && !MSFS_DYNAMIC
        &msfs_vfsops,           "advfs",	/* 10 = MOUNT_MSFS */
#else
        (struct vfsops *)0,     "",		/* 10 = MOUNT_MSFS */
#endif
#if	FFM_FS && !FFM_FS_DYNAMIC
	&ffm_vfsops,		"ffm",		/* 11 = MOUNT_FFM */
#else
	(struct vfsops *)0,	"",		/* 11 = MOUNT_FFM */
#endif
	&fdfs_vfsops,		"fdfs",		/* 12 = MOUNT_FDFS */
        (struct vfsops *)0,     "",		/* 13 = MOUNT_ADDON */
#if     NFS && !NFS_DYNAMIC
        &nfs3_vfsops,           "nfsv3", 	/* 14 = MOUNT_NFS3 */
#else
        (struct vfsops *)0,     "",		/* 14 = MOUNT_NFS3 */
#endif
#if defined(CFS) && CFS
	&cfs_vfsops,		"cfs",		/* 15 = MOUNT_CFS */
#else
        (struct vfsops *)0,     "",		/* 15 = MOUNT_CFS */
#endif
#if defined(CSPECFS) && CSPECFS
	&cspecvfsops,		"cspec",	/* 16 = MOUNT_CSPEC */
#else
        (struct vfsops *)0,     "",		/* 16 = MOUNT_CSPEC */
#endif
#if     DVDFS && !DVDFS_DYNAMIC
        &dvdfs_vfsops,           "dvdfs", 	/* 17 = MOUNT_DVDFS */
#else
        (struct vfsops *)0,     "",		/* 17 = MOUNT_DVDFS */
#endif
#if defined(AUTOFS) && AUTOFS
        &autofs_vfsops,         "autofs",	/* 18 = MOUNT_AUTOFS */
#else
        (struct vfsops *)0,     "",		/* 18 = MOUNT_AUTOFS */
#endif
};

int mount_maxtype = sizeof (vfssw) / sizeof (vfssw[0]) - 1;

/*
 * Configuration parameters for vnode deallocation that are not exported 
 * to the config file. Can be changed on a running kernel, via dbx to 
 * tune the system.  
 *
 * This structure and declaration are here to allow these config values
 * to be different on clustered vs non-clustered systems, without
 * replacing the vfs.mod module.
 *
 * Note that the definition of vn_conf is duplicated in vfs_subr.c, and
 * should be updated if this structure is updated.
 */
struct vn_conf {
  u_int  percent_mem_for_vnodes;/* % of system memory for vnodes */
  u_int  dealloc_percent;       /* target %  of free vnodes for dealloc */
  u_int  max_deallocs_per_try;  /* limits deallocs per try */
  u_int  max_failed_deallocs;   /* stop deallocs if failures exceed this val */
  u_int  min_dealloc_interval;  /* minimum interval between vnode deallocs */
  u_int  max_vdealloc_recursions; /* to check resursions */
};

#if defined(CFS) && CFS
struct vn_conf vn_conf = { 4, 40, 3, 20, 10, 0};
#else
struct vn_conf vn_conf = { 5, 40, 3, 20, 10, 5};
#endif

/*
 * Allow adjustment of the maximum number of locks that may be held.
 * The space required is allocated as necessary but is limited so a
 * wayward program can't eat up all of kernel memory with lock structures.
 * nlock_record may be overridden by something like
 * "options nlock_record = 20000" in the config file.
 */
#ifndef NLOCK_RECORD
#define NLOCK_RECORD 10000
#endif
long nlock_record = NLOCK_RECORD;

void 
vfs_mountroot(void)
{
	register int error;
	extern int networkboot;
	extern int mdsize;

	/*
	 * Allocate a mount structure for root (rootfs).
	 * Initialize locks and common fields
	 */
	MALLOC(rootfs, struct mount *, sizeof(struct mount), M_MOUNT, M_WAITOK);
	bzero(rootfs, sizeof(*rootfs));
	UNMOUNT_LOCK_INIT(rootfs);
	MOUNT_VLIST_LOCK_INIT(rootfs);
	MOUNT_LOCK_INIT(rootfs);
	MOUNT_THROTTLE_LOCK_INIT(rootfs);

	/*
	 * Set up string for root.
	 */
	rootmpname = (char *) srootmpname;

	/*
	 * Select the appropriate root vfsops vector.
	 */
	if (mdsize > 0)
		rootfs->m_op = vfssw[MOUNT_UFS].vops_ptr;
	else if (networkboot)
		rootfs->m_op = vfssw[MOUNT_NFS].vops_ptr;
	else if (vfssw[MOUNT_CFS].vops_ptr)
		rootfs->m_op = vfssw[MOUNT_CFS].vops_ptr;
	else
		rootfs->m_op = vfssw[MOUNT_UFS].vops_ptr;

        rootfs->m_nxt = rootfs;
        rootfs->m_prev = rootfs;
	/*
	 * Call the filesystem specific mountroot routine.
	 */
	VFS_MOUNTROOT(rootfs, &rootvp, error);
	if (networkboot && error)
		panic("vfs_mountroot: cannot mount root");
	if (error && vfssw[MOUNT_MSFS].vops_ptr) {
		rootfs->m_op = vfssw[MOUNT_MSFS].vops_ptr;
		VFS_MOUNTROOT(rootfs, &rootvp, error);
	}
#if	CDFS && !CDFS_DYNAMIC
	if (error && vfssw[MOUNT_CDFS].vops_ptr) {
		rootfs->m_op = vfssw[MOUNT_CDFS].vops_ptr;
		VFS_MOUNTROOT(rootfs, &rootvp, error);
	}
#endif	/* CDFS && !CDFS_DYNAMIC */
	if (error)
		panic("vfs_mountroot: cannot mount root");


	/*
         * Get vnode for '/'.
         * Setup rootdir and u.u_cdir to point to it.
         */
mounted:
        rootvp->v_mount = rootfs;       /* set up root device's mount ptr */
        VFS_ROOT(rootfs, &rootdir, error);
        if (error)
                panic("vfs_mountroot: cannot get root filesystem");
	VFS_START(rootfs, 0, error);
        if (error)
                panic("vfs_mountroot: cannot start root filesystem");
        u.u_cdir = rootdir;
        VREF(u.u_cdir);
        u.u_rdir = NULL;
}

#include <quota.h>
/*
 * operate on filesystem quotas
 */
/* ARGSUSED */
int 
quotactl(
	struct proc *p,
	void *args,
	long *retval)
{
#if defined(QUOTA) && QUOTA
	register struct args {
		char *path;
		long cmd;
		long uid;
		caddr_t arg;
	} *uap = (struct args *) args;
	register struct mount *mp;
	register struct nameidata *ndp = &u.u_nd;
	struct vnode *vp;
	int error;

	/*
	 * namei will pend at the filesystem's root if
	 * an unmount is in progress so a quotactl operation
	 * can't interfere with a pending unmount.  However,
	 * an unmount can begin after a quotactl operation starts,
	 * so the filesystem-specific unmount routine must handle
	 * potential unmount/quotactl races.
	 *
	 * We defer the vrele until after the quota operation
	 * so that we may hang onto our reference on the filesystem
	 * across the lower layer's operation.  Otherwise, the
	 * filesystem could become unmounted in the normal way while
	 * we do our thing.  (Forcible unmount is another case entirely.)
	 */
	ndp->ni_nameiop = LOOKUP | FOLLOW;
	ndp->ni_segflg = UIO_USERSPACE;
	ndp->ni_dirp = uap->path;
	if (error = namei(ndp))
		return (error);
	mp = ndp->ni_vp->v_mount;
	vp = ndp->ni_vp;
	VFS_QUOTACTL(mp, uap->cmd, uap->uid, uap->arg, error);
	vrele(vp);
	return (error);
#else
	return(EOPNOTSUPP);
#endif
}

#if !NFS
/*
 * All routines/code necessary for the kernel to build when
 * the NFS server is not configured should be placed in nfs_server_stubs.c
 *
 * All routines/code necessary for the kernel to build when
 * the NFS client is not configured should be placed here.
 */

nfs_async_daemon = NULL;
struct vfs_ubcops nfs_ubcops, nfs3_ubcops;

#endif /* NFS */
